@@ -1,6 +1,9 @@
 # Orca
 #
-# Copyright 2010 Joanmarie Diggs.
+# Copyright (C) 2010 Joanmarie Diggs
+# Copyright (C) 2011 Igalia, S.L.
+#
+# Author: Joanmarie Diggs <jdiggs@igalia.com>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -20,12 +23,15 @@
 __id__ = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
-__copyright__ = "Copyright (c) 2010 Joanmarie Diggs."
+__copyright__ = "Copyright (c) 2010 Joanmarie Diggs." \
+                "Copyright (c) 2011 Igalia, S.L."
 __license__   = "LGPL"
 
 import pyatspi
+import re
 
 import orca.script_utilities as script_utilities
+import orca.settings as settings
 
 #############################################################################
 #                                                                           #
@@ -85,57 +91,6 @@ class Utilities(script_utilities.Utilities):
 
         return readOnly
 
-    def adjustForLinks(self, obj, line, startOffset):
-        """Adjust line to include the word "link" after any hypertext links.
-        Overridden here to deal with parents containing children which in
-        turn contain the links and implement the hypertext interface.
-
-        Arguments:
-        - obj: the accessible object that this line came from.
-        - line: the string to adjust for links.
-        - startOffset: the caret offset at the start of the line.
-
-        Returns: a new line adjusted to add the speaking of "link" after
-        text which is also a link.
-        """
-
-        adjustedLine = script_utilities.Utilities.adjustForLinks(
-                self, obj, line, startOffset)
-
-        roles = [pyatspi.ROLE_LIST_ITEM]
-        if not obj.getRole() in roles or not obj.childCount:
-            return adjustedLine
-
-        child = obj[0]
-        try:
-            hypertext = obj.queryHypertext()
-            nLinks = hypertext.getNLinks()
-        except NotImplementedError:
-            nLinks = 0
-
-        if not nLinks:
-            try:
-                hypertext = child.queryHypertext()
-                nLinks = hypertext.getNLinks()
-            except NotImplementedError:
-                pass
-
-        if not nLinks:
-            return adjustedLine
-
-        try:
-            objText = obj.queryText()
-            childText = child.queryText()
-        except NotImplementedError:
-            return adjustedLine
-
-        adjustment = objText.characterCount - childText.characterCount
-        if adjustment:
-            adjustedLine = script_utilities.Utilities.adjustForLinks(
-                self, child, line, startOffset - adjustment)
-
-        return adjustedLine
-
     def displayedText(self, obj):
         """Returns the text being displayed for an object.
 
@@ -150,9 +105,59 @@ class Utilities(script_utilities.Utilities):
         if text and text != self.EMBEDDED_OBJECT_CHARACTER:
             return text
 
-        if obj.getRole() == pyatspi.ROLE_LINK:
+        if obj.getRole() in [pyatspi.ROLE_LINK, pyatspi.ROLE_LIST_ITEM]:
             text = ' '.join(map(self.displayedText, (x for x in obj)))
             if not text:
                 text = self.linkBasename(obj)
 
         return text
+
+    def getObjectsFromEOCs(self, obj, boundary=None, offset=None):
+        """Breaks the string containing a mixture of text and embedded object
+        characters into a list of (obj, startOffset, endOffset, string) tuples.
+
+        Arguments
+        - obj: the object whose EOCs we need to expand into tuples
+        - boundary: the pyatspi text boundary type. If None, get all text.
+        - offset: the character offset. If None, use the current offset.
+
+        Returns a list of (obj, startOffset, endOffset, string) tuples.
+        """
+
+        try:
+            text = obj.queryText()
+            htext = obj.queryHypertext()
+        except (AttributeError, NotImplementedError):
+            return [(obj, 0, 1, '')]
+
+        string = text.getText(0, -1).decode('UTF-8')
+        if not string:
+            return [(obj, 0, 1, '')]
+
+        if offset == None:
+            offset = text.caretOffset
+        if boundary == None:
+            start = 0
+            end = text.characterCount
+        else:
+            if boundary == pyatspi.TEXT_BOUNDARY_CHAR:
+                key, mods = self.lastKeyAndModifiers()
+                if (mods & settings.SHIFT_MODIFIER_MASK) and key == 'Right':
+                    offset -= 1
+            segment, start, end = text.getTextAtOffset(offset, boundary)
+
+        pattern = re.compile(self.EMBEDDED_OBJECT_CHARACTER)
+        offsets = [m.start(0) for m in re.finditer(pattern, string)]
+        offsets = filter(lambda x: start <= x < end, offsets)
+
+        objects = []
+        objs = [obj[htext.getLinkIndex(offset)] for offset in offsets]
+        ranges = [self.getHyperlinkRange(x) for x in objs]
+        for i, (first, last) in enumerate(ranges):
+            objects.append((obj, start, first, string[start:first]))
+            objects.append((objs[i], first, last, ''))
+            start = last
+        objects.append((obj, start, end, string[start:end]))
+        objects = filter(lambda x: x[1] < x[2], objects)
+
+        return objects
